#!/usr/bin/env python3
"""
Monitors external events like trades made outside the bot and price alarms.
"""

import logging
import asyncio
from datetime import datetime, timedelta, timezone
from typing import Optional, Dict, Any, List

# Assuming AlarmManager will be moved here or imported appropriately
# from .alarm_manager import AlarmManager 
from src.monitoring.alarm_manager import AlarmManager # Keep if AlarmManager stays in its own file as per original structure
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class ExternalEventMonitor:
    def __init__(self, trading_engine, notification_manager, alarm_manager, market_monitor_cache, shared_state):
        self.trading_engine = trading_engine
        self.notification_manager = notification_manager
        self.alarm_manager = alarm_manager
        self.market_monitor_cache = market_monitor_cache
        self.shared_state = shared_state # Expected to contain {'external_stop_losses': ...}
        self.last_processed_trade_time: Optional[datetime] = None
        # Add necessary initializations, potentially loading last_processed_trade_time

    async def _check_price_alarms(self):
        """Check price alarms and trigger notifications."""
        try:
            active_alarms = self.alarm_manager.get_all_active_alarms()
            
            if not active_alarms:
                return
            
            tokens_to_check = list(set(alarm['token'] for alarm in active_alarms))
            
            for token in tokens_to_check:
                try:
                    symbol = f"{token}/USDC:USDC"
                    market_data = self.trading_engine.get_market_data(symbol)
                    
                    if not market_data or not market_data.get('ticker'):
                        continue
                    
                    current_price = float(market_data['ticker'].get('last', 0))
                    if current_price <= 0:
                        continue
                    
                    token_alarms = [alarm for alarm in active_alarms if alarm['token'] == token]
                    
                    for alarm in token_alarms:
                        target_price = alarm['target_price']
                        direction = alarm['direction']
                        
                        should_trigger = False
                        if direction == 'above' and current_price >= target_price:
                            should_trigger = True
                        elif direction == 'below' and current_price <= target_price:
                            should_trigger = True
                        
                        if should_trigger:
                            triggered_alarm = self.alarm_manager.trigger_alarm(alarm['id'], current_price)
                            if triggered_alarm:
                                await self._send_alarm_notification(triggered_alarm)
                
                except Exception as e:
                    logger.error(f"Error checking alarms for {token}: {e}")
                    
        except Exception as e:
            logger.error("❌ Error checking price alarms: {e}")
    
    async def _send_alarm_notification(self, alarm: Dict[str, Any]):
        """Send notification for triggered alarm."""
        try:
            if self.notification_manager:
                await self.notification_manager.send_alarm_triggered_notification(
                    alarm['token'], 
                    alarm['target_price'], 
                    alarm['triggered_price'], 
                    alarm['direction']
                )
            else:
                logger.info(f"🔔 ALARM TRIGGERED: {alarm['token']} @ ${alarm['triggered_price']:,.2f}")
            
        except Exception as e:
            logger.error(f"❌ Error sending alarm notification: {e}")
    
    async def _check_external_trades(self):
        """Check for trades made outside the Telegram bot and update stats."""
        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                logger.warning("TradingStats not available in _check_external_trades. Skipping.")
                return

            external_trades_processed = 0
            symbols_with_fills = set()

            recent_fills = self.trading_engine.get_recent_fills()
            if not recent_fills:
                logger.debug("No recent fills data available")
                return

            if not hasattr(self, 'last_processed_trade_time') or self.last_processed_trade_time is None:
                try:
                    # Ensure this metadata key is the one used by MarketMonitor for saving this state.
                    last_time_str = stats._get_metadata('market_monitor_last_processed_trade_time') 
                    if last_time_str:
                        self.last_processed_trade_time = datetime.fromisoformat(last_time_str).replace(tzinfo=timezone.utc)
                    else:
                        self.last_processed_trade_time = datetime.now(timezone.utc) - timedelta(hours=1)
                except Exception: 
                     self.last_processed_trade_time = datetime.now(timezone.utc) - timedelta(hours=1)

            for fill in recent_fills:
                try:
                    trade_id = fill.get('id')
                    timestamp_ms = fill.get('timestamp')
                    symbol_from_fill = fill.get('symbol')
                    side_from_fill = fill.get('side')
                    amount_from_fill = float(fill.get('amount', 0))
                    price_from_fill = float(fill.get('price', 0))
                    
                    timestamp_dt = datetime.fromtimestamp(timestamp_ms / 1000, tz=timezone.utc) if timestamp_ms else datetime.now(timezone.utc)
                    
                    if self.last_processed_trade_time and timestamp_dt <= self.last_processed_trade_time:
                        continue
                    
                    fill_processed_this_iteration = False
                    
                    if not (symbol_from_fill and side_from_fill and amount_from_fill > 0 and price_from_fill > 0):
                        logger.warning(f"Skipping fill with incomplete data: {fill}")
                        continue

                    full_symbol = symbol_from_fill
                    token = symbol_from_fill.split('/')[0] if '/' in symbol_from_fill else symbol_from_fill.split(':')[0]

                    exchange_order_id_from_fill = fill.get('info', {}).get('oid')

                    if exchange_order_id_from_fill:
                        pending_lc = stats.get_lifecycle_by_entry_order_id(exchange_order_id_from_fill, status='pending')
                        if pending_lc and pending_lc.get('symbol') == full_symbol:
                            success = stats.update_trade_position_opened(
                                lifecycle_id=pending_lc['trade_lifecycle_id'],
                                entry_price=price_from_fill,
                                entry_amount=amount_from_fill,
                                exchange_fill_id=trade_id
                            )
                            if success:
                                logger.info(f"📈 Lifecycle ENTRY: {pending_lc['trade_lifecycle_id']} for {full_symbol} updated by fill {trade_id}.")
                                symbols_with_fills.add(token)
                                order_in_db_for_entry = stats.get_order_by_exchange_id(exchange_order_id_from_fill)
                                if order_in_db_for_entry:
                                    stats.update_order_status(order_db_id=order_in_db_for_entry['id'], new_status='filled', amount_filled_increment=amount_from_fill)
                            fill_processed_this_iteration = True
                    
                    if not fill_processed_this_iteration:
                        active_lc = None
                        closure_reason_action_type = None
                        bot_order_db_id_to_update = None

                        if exchange_order_id_from_fill:
                            bot_order_for_fill = stats.get_order_by_exchange_id(exchange_order_id_from_fill)
                            if bot_order_for_fill and bot_order_for_fill.get('symbol') == full_symbol:
                                order_type = bot_order_for_fill.get('type')
                                order_side = bot_order_for_fill.get('side')
                                if order_type == 'market':
                                    potential_lc = stats.get_trade_by_symbol_and_status(full_symbol, 'position_opened')
                                    if potential_lc:
                                        lc_pos_side = potential_lc.get('position_side')
                                        if (lc_pos_side == 'long' and order_side == 'sell' and side_from_fill == 'sell') or \
                                           (lc_pos_side == 'short' and order_side == 'buy' and side_from_fill == 'buy'):
                                            active_lc = potential_lc
                                            closure_reason_action_type = f"bot_exit_{lc_pos_side}_close"
                                            bot_order_db_id_to_update = bot_order_for_fill.get('id')
                                            logger.info(f"ℹ️ Lifecycle BOT EXIT: Fill {trade_id} (OID {exchange_order_id_from_fill}) for {full_symbol} matches bot exit for lifecycle {active_lc['trade_lifecycle_id']}.")
                                
                                if not active_lc:
                                    lc_by_sl = stats.get_lifecycle_by_sl_order_id(exchange_order_id_from_fill, status='position_opened')
                                    if lc_by_sl and lc_by_sl.get('symbol') == full_symbol:
                                        active_lc = lc_by_sl
                                        closure_reason_action_type = f"sl_{active_lc.get('position_side')}_close"
                                        bot_order_db_id_to_update = bot_order_for_fill.get('id') 
                                        logger.info(f"ℹ️ Lifecycle SL: Fill {trade_id} for OID {exchange_order_id_from_fill} matches SL for lifecycle {active_lc['trade_lifecycle_id']}.")
                                    
                                    if not active_lc: 
                                        lc_by_tp = stats.get_lifecycle_by_tp_order_id(exchange_order_id_from_fill, status='position_opened')
                                        if lc_by_tp and lc_by_tp.get('symbol') == full_symbol:
                                            active_lc = lc_by_tp
                                            closure_reason_action_type = f"tp_{active_lc.get('position_side')}_close"
                                            bot_order_db_id_to_update = bot_order_for_fill.get('id') 
                                            logger.info(f"ℹ️ Lifecycle TP: Fill {trade_id} for OID {exchange_order_id_from_fill} matches TP for lifecycle {active_lc['trade_lifecycle_id']}.")

                        if not active_lc:
                            potential_lc_external = stats.get_trade_by_symbol_and_status(full_symbol, 'position_opened')
                            if potential_lc_external:
                                lc_pos_side = potential_lc_external.get('position_side')
                                if (lc_pos_side == 'long' and side_from_fill == 'sell') or \
                                   (lc_pos_side == 'short' and side_from_fill == 'buy'):
                                    active_lc = potential_lc_external
                                    closure_reason_action_type = f"external_{lc_pos_side}_close"
                                    logger.info(f"ℹ️ Lifecycle EXTERNAL CLOSE: Fill {trade_id} for {full_symbol} (no matching bot OID) for lifecycle {active_lc['trade_lifecycle_id']}.")
                        
                        if active_lc and closure_reason_action_type:
                            lc_id = active_lc['trade_lifecycle_id']
                            lc_entry_price = active_lc.get('entry_price', 0)
                            lc_position_side = active_lc.get('position_side')

                            realized_pnl = 0
                            if lc_position_side == 'long':
                                realized_pnl = amount_from_fill * (price_from_fill - lc_entry_price)
                            elif lc_position_side == 'short':
                                realized_pnl = amount_from_fill * (lc_entry_price - price_from_fill)
                            
                            success = stats.update_trade_position_closed(
                                lifecycle_id=lc_id, exit_price=price_from_fill,
                                realized_pnl=realized_pnl, exchange_fill_id=trade_id
                            )
                            if success:
                                pnl_emoji = "🟢" if realized_pnl >= 0 else "🔴"
                                formatter = get_formatter()
                                logger.info(f"{pnl_emoji} Lifecycle CLOSED: {lc_id} ({closure_reason_action_type}). PNL for fill: {formatter.format_price_with_symbol(realized_pnl)}")
                                symbols_with_fills.add(token)
                                if self.notification_manager:
                                    await self.notification_manager.send_external_trade_notification(
                                        full_symbol, side_from_fill, amount_from_fill, price_from_fill, 
                                        closure_reason_action_type, timestamp_dt.isoformat()
                                    )
                                stats._migrate_trade_to_aggregated_stats(lc_id)
                                if bot_order_db_id_to_update:
                                    stats.update_order_status(order_db_id=bot_order_db_id_to_update, new_status='filled', amount_filled_increment=amount_from_fill)
                                fill_processed_this_iteration = True

                    if not fill_processed_this_iteration:
                        # Access external_stop_losses via self.shared_state
                        if (exchange_order_id_from_fill and 
                            self.shared_state.get('external_stop_losses') and
                            exchange_order_id_from_fill in self.shared_state['external_stop_losses']):
                            stop_loss_info = self.shared_state['external_stop_losses'][exchange_order_id_from_fill]
                            formatter = get_formatter()
                            logger.info(f"🛑 External SL (MM Tracking): {token} Order {exchange_order_id_from_fill} filled @ {formatter.format_price_with_symbol(price_from_fill, token)}")
                            
                            sl_active_lc = stats.get_trade_by_symbol_and_status(full_symbol, 'position_opened')
                            if sl_active_lc:
                                lc_id = sl_active_lc['trade_lifecycle_id']
                                lc_entry_price = sl_active_lc.get('entry_price', 0)
                                lc_pos_side = sl_active_lc.get('position_side')
                                realized_pnl = amount_from_fill * (price_from_fill - lc_entry_price) if lc_pos_side == 'long' else amount_from_fill * (lc_entry_price - price_from_fill)
                                
                                success = stats.update_trade_position_closed(lc_id, price_from_fill, realized_pnl, trade_id)
                                if success:
                                    pnl_emoji = "🟢" if realized_pnl >= 0 else "🔴"
                                    logger.info(f"{pnl_emoji} Lifecycle CLOSED by External SL (MM): {lc_id}. PNL: {formatter.format_price_with_symbol(realized_pnl)}")
                                    if self.notification_manager:
                                        await self.notification_manager.send_stop_loss_execution_notification(
                                            stop_loss_info, full_symbol, side_from_fill, amount_from_fill, price_from_fill, 
                                            f'{lc_pos_side}_closed_external_sl', timestamp_dt.isoformat(), realized_pnl
                                        )
                                    stats._migrate_trade_to_aggregated_stats(lc_id)
                                    # Modify shared state carefully
                                    if exchange_order_id_from_fill in self.shared_state['external_stop_losses']:
                                        del self.shared_state['external_stop_losses'][exchange_order_id_from_fill]
                                    fill_processed_this_iteration = True
                            else:
                                logger.warning(f"⚠️ External SL (MM) {exchange_order_id_from_fill} for {full_symbol}, but no active lifecycle found.")
                    
                    if not fill_processed_this_iteration:
                        existing_open_lc = stats.get_trade_by_symbol_and_status(full_symbol, 'position_opened')
                        if existing_open_lc:
                            lc_id = existing_open_lc['trade_lifecycle_id']
                            lc_entry_price = existing_open_lc.get('entry_price', 0)
                            lc_position_side = existing_open_lc.get('position_side')
                            lc_current_size_before_fill = existing_open_lc.get('current_position_size', 0)

                            is_potentially_closing_external_fill = False
                            if lc_position_side == 'long' and side_from_fill.lower() == 'sell':
                                is_potentially_closing_external_fill = True
                            elif lc_position_side == 'short' and side_from_fill.lower() == 'buy':
                                is_potentially_closing_external_fill = True
                            
                            if is_potentially_closing_external_fill:
                                logger.info(f"ℹ️ Detected potentially closing external fill {trade_id} for {full_symbol} (Lifecycle: {lc_id}). Verifying exchange position state...")
                                
                                fresh_positions_after_fill = self.trading_engine.get_positions() or []
                                position_on_exchange_after_fill = None
                                for pos in fresh_positions_after_fill:
                                    if pos.get('symbol') == full_symbol:
                                        position_on_exchange_after_fill = pos
                                        break
                                
                                position_is_closed_on_exchange = False
                                if position_on_exchange_after_fill is None:
                                    position_is_closed_on_exchange = True
                                    logger.info(f"✅ Exchange Verification: Position for {full_symbol} (Lifecycle: {lc_id}) not found after fill {trade_id}. Confirming closure.")
                                elif abs(float(position_on_exchange_after_fill.get('contracts', 0))) < 1e-9:
                                    position_is_closed_on_exchange = True
                                    logger.info(f"✅ Exchange Verification: Position for {full_symbol} (Lifecycle: {lc_id}) has zero size on exchange after fill {trade_id}. Confirming closure.")

                                if position_is_closed_on_exchange:
                                    amount_for_pnl_calc = amount_from_fill
                                    if abs(lc_current_size_before_fill - amount_from_fill) < 0.000001 * amount_from_fill:
                                         amount_for_pnl_calc = lc_current_size_before_fill

                                    logger.info(f"ℹ️ Attempting to close lifecycle {lc_id} for {full_symbol} via confirmed external fill {trade_id}.")
                                    realized_pnl = 0
                                    if lc_position_side == 'long':
                                        realized_pnl = amount_for_pnl_calc * (price_from_fill - lc_entry_price)
                                    elif lc_position_side == 'short':
                                        realized_pnl = amount_for_pnl_calc * (lc_entry_price - price_from_fill)
                                    
                                    success = stats.update_trade_position_closed(
                                        lifecycle_id=lc_id, 
                                        exit_price=price_from_fill, 
                                        realized_pnl=realized_pnl, 
                                        exchange_fill_id=trade_id
                                    )
                                    if success:
                                        pnl_emoji = "🟢" if realized_pnl >= 0 else "🔴"
                                        formatter = get_formatter()
                                        logger.info(f"{pnl_emoji} Lifecycle CLOSED (Verified External): {lc_id}. PNL for fill: {formatter.format_price_with_symbol(realized_pnl)}")
                                        symbols_with_fills.add(token)
                                        if self.notification_manager:
                                            await self.notification_manager.send_external_trade_notification(
                                                full_symbol, side_from_fill, amount_from_fill, price_from_fill,
                                                f"verified_external_{lc_position_side}_close",
                                                timestamp_dt.isoformat()
                                            )
                                        stats._migrate_trade_to_aggregated_stats(lc_id)
                                        fill_processed_this_iteration = True
                                    else:
                                        logger.error(f"❌ Failed to close lifecycle {lc_id} via verified external fill {trade_id}.")
                                else:
                                    current_size_on_exchange = float(position_on_exchange_after_fill.get('contracts', 0)) if position_on_exchange_after_fill else 'Unknown'
                                    logger.warning(f"⚠️ External fill {trade_id} for {full_symbol} (Lifecycle: {lc_id}, Amount: {amount_from_fill}) did NOT fully close position. Exchange size now: {current_size_on_exchange}. Lifecycle remains open. Fill will be recorded as 'external_unmatched'.")

                    if not fill_processed_this_iteration:
                        # Check if this external trade opens a new position that should be tracked
                        current_positions_from_cache_map = {
                            pos.get('symbol'): pos for pos in (self.market_monitor_cache.cached_positions or [])
                            if pos.get('symbol') and abs(float(pos.get('contracts', 0))) > 1e-9
                        }
                        
                        # Check if there's a position on the exchange for this symbol
                        exchange_position = current_positions_from_cache_map.get(full_symbol)
                        should_create_lifecycle = False
                        
                        if exchange_position:
                            # There's a position on exchange but no lifecycle tracking it
                            # This indicates a new external position that should be tracked
                            should_create_lifecycle = True
                            logger.info(f"🚀 Detected new external position for {full_symbol}: {side_from_fill} {amount_from_fill} @ {price_from_fill}")
                        
                        if should_create_lifecycle:
                            # Create a new trade lifecycle for this external position
                            lifecycle_id = stats.create_trade_lifecycle(
                                symbol=full_symbol,
                                side=side_from_fill,
                                entry_order_id=exchange_order_id_from_fill,
                                trade_type="external"
                            )
                            
                            if lifecycle_id:
                                # Update the lifecycle to position_opened status
                                success = stats.update_trade_position_opened(
                                    lifecycle_id=lifecycle_id,
                                    entry_price=price_from_fill,
                                    entry_amount=amount_from_fill,
                                    exchange_fill_id=trade_id
                                )
                                
                                if success:
                                    logger.info(f"📈 Created and opened new external lifecycle: {lifecycle_id[:8]} for {full_symbol}")
                                    symbols_with_fills.add(token)
                                    
                                    # Send notification for new external position
                                    if self.notification_manager:
                                        await self.notification_manager.send_external_trade_notification(
                                            full_symbol, side_from_fill, amount_from_fill, price_from_fill,
                                            "position_opened", timestamp_dt.isoformat()
                                        )
                                    fill_processed_this_iteration = True
                        
                        # If we didn't create a lifecycle, fall back to the old behavior
                        if not fill_processed_this_iteration:
                            all_open_positions_in_db = stats.get_open_positions()
                            db_open_symbols = {pos_db.get('symbol') for pos_db in all_open_positions_in_db}

                            if full_symbol in db_open_symbols:
                                logger.error(f"🚨 DIAGNOSTIC: Contradiction for {full_symbol}! get_open_positions() includes it, but get_trade_by_symbol_and_status('{full_symbol}', 'position_opened') failed to find it within _check_external_trades context for fill {trade_id}. This needs investigation into TradingStats symbol querying.")
                            
                            potential_match_failure_logged = False
                            if not stats.get_trade_by_symbol_and_status(full_symbol, 'position_opened'):
                                logger.warning(f"⚠️ DIAGNOSTIC for UNMATCHED FILL {trade_id} ({full_symbol}):")
                                logger.warning(f"   Fill details: Side={side_from_fill}, Amount={amount_from_fill}, Price={price_from_fill}")
                                logger.warning(f"   Attempted lookup with full_symbol='{full_symbol}' and status='position_opened' found NO active lifecycle.")
                                if all_open_positions_in_db:
                                    logger.warning(f"   However, DB currently has these 'position_opened' lifecycles (symbol - lifecycle_id):")
                                    for db_pos in all_open_positions_in_db:
                                        logger.warning(f"     - '{db_pos.get('symbol')}' - ID: {db_pos.get('trade_lifecycle_id')}")
                                    base_token_fill = full_symbol.split('/')[0].split(':')[0]
                                    near_matches = [db_s for db_s in db_open_symbols if base_token_fill in db_s]
                                    if near_matches:
                                        logger.warning(f"   Possible near matches in DB for base token '{base_token_fill}': {near_matches}")
                                    else:
                                        logger.warning(f"   No near matches found in DB for base token '{base_token_fill}'.")
                                else:
                                    logger.warning("   DB has NO 'position_opened' lifecycles at all right now.")
                                potential_match_failure_logged = True
                            
                            linked_order_db_id = None
                            if exchange_order_id_from_fill:
                                order_in_db = stats.get_order_by_exchange_id(exchange_order_id_from_fill)
                                if order_in_db:
                                    linked_order_db_id = order_in_db.get('id')
                                    logger.info(f"🔗 Fallback: Fill {trade_id} for OID {exchange_order_id_from_fill} (DB ID {linked_order_db_id}) not tied to active lifecycle step.")
                                    current_status = order_in_db.get('status', '')
                                    if current_status in ['open', 'partially_filled', 'pending_submission']:
                                        amt_req = float(order_in_db.get('amount_requested', 0))
                                        amt_filled_so_far = float(order_in_db.get('amount_filled',0))
                                        new_status = 'partially_filled'
                                        if (amt_filled_so_far + amount_from_fill) >= amt_req - 1e-9:
                                            new_status = 'filled'
                                        stats.update_order_status(
                                            order_db_id=linked_order_db_id, new_status=new_status,
                                            amount_filled_increment=amount_from_fill
                                        )
                                        logger.info(f"📊 Updated bot order {linked_order_db_id} (fallback): {current_status} → {new_status}")
                            
                            if not (hasattr(stats, 'get_trade_by_exchange_fill_id') and stats.get_trade_by_exchange_fill_id(trade_id)):
                                stats.record_trade(
                                    full_symbol, side_from_fill, amount_from_fill, price_from_fill, 
                                    exchange_fill_id=trade_id, trade_type="external_unmatched",
                                    timestamp=timestamp_dt.isoformat(),
                                    linked_order_table_id_to_link=linked_order_db_id 
                                )
                                logger.info(f"📋 Recorded trade via FALLBACK: {trade_id} (Unmatched External Fill)")
                                
                                # Send notification for unmatched external trade
                                if self.notification_manager:
                                    await self.notification_manager.send_external_trade_notification(
                                        full_symbol, side_from_fill, amount_from_fill, price_from_fill,
                                        "external_unmatched", timestamp_dt.isoformat()
                                    )
                            fill_processed_this_iteration = True

                    if fill_processed_this_iteration:
                        external_trades_processed += 1
                        if self.last_processed_trade_time is None or timestamp_dt > self.last_processed_trade_time:
                           self.last_processed_trade_time = timestamp_dt
                        
                except Exception as e:
                    logger.error(f"Error processing fill {fill.get('id', 'N/A')}: {e}", exc_info=True)
                    continue
            
            if external_trades_processed > 0:
                stats._set_metadata('market_monitor_last_processed_trade_time', self.last_processed_trade_time.isoformat())
                logger.info(f"💾 Saved MarketMonitor state (last_processed_trade_time) to DB: {self.last_processed_trade_time.isoformat()}")
                logger.info(f"📊 Processed {external_trades_processed} external trades")
                if symbols_with_fills:
                    logger.info(f"ℹ️ Symbols with processed fills this cycle: {list(symbols_with_fills)}")
                
        except Exception as e:
            logger.error(f"❌ Error checking external trades: {e}", exc_info=True) 